//------------------------------------------------------------------------------
// This file is part of the OpenStructure project <www.openstructure.org>
//
// Copyright (C) 2008-2020 by the OpenStructure authors
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation; either version 3.0 of the License, or (at your option)
// any later version.
// This library is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this library; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//------------------------------------------------------------------------------
#ifndef OST_GFX_RENDER_MODE_HH
#define OST_GFX_RENDER_MODE_HH

/*
  constants used for all rendering modes

  Author: Ansgar Philippsen
*/

namespace ost { namespace gfx {

namespace RenderMode {

enum Type {
  DEBUG,         // for all
  DEBUG2,        // for all
  TEST,          // for all
  TEST2,         // for all
  SIMPLE,        // entity connect
  CUSTOM,        // entity connect
  CPK,           // entity connect
  LINE_TRACE,    // entity trace
  TRACE,         // trace
  SLINE,         // entity trace
  TUBE,          // entity trace
  HSC,           // entity trace
  FILL,          // iso map
  OUTLINE        // iso map
};

}

}} // ns

#endif
