//------------------------------------------------------------------------------
// This file is part of the OpenStructure project <www.openstructure.org>
//
// Copyright (C) 2008-2020 by the OpenStructure authors
// Copyright (C) 2003-2010 by the IPLT authors
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation; either version 3.0 of the License, or (at your option)
// any later version.
// This library is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this library; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//------------------------------------------------------------------------------
#ifndef INVERTED_MASK_HH_
#define INVERTED_MASK_HH_

#include "mask_base.hh"

namespace ost {
namespace img
{

class DLLEXPORT_OST_IMG_BASE InvertedMask: public MaskBase
{
public:
  InvertedMask(const MaskPtr& lhs);
  InvertedMask(const InvertedMask& m);
  virtual ~InvertedMask();
  virtual MaskPtr Clone();
  virtual bool IsInside(const Vec2& v);

  virtual void Shift(const Vec2& v);
  virtual void Expand(Real d);
  virtual void Scale(Real d);
  
  virtual void Apply(MaskVisitor& v);

protected:
  MaskPtr lhs_;
};


}}

#endif /*INVERTED_MASK_HH_*/
