//------------------------------------------------------------------------------
// This file is part of the OpenStructure project <www.openstructure.org>
//
// Copyright (C) 2008-2020 by the OpenStructure authors
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation; either version 3.0 of the License, or (at your option)
// any later version.
// This library is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this library; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//------------------------------------------------------------------------------

/*
  Author: Stefan Scheuber
*/
#ifndef OST_GUI_SPNAV_INPUT_HH
#define OST_GUI_SPNAV_INPUT_HH


#include <ost/geom/geom.hh>
#include <ost/gui/module_config.hh>

#include <QThread>
#include <QMetaType>
Q_DECLARE_METATYPE(geom::Mat4);

namespace ost { namespace gui {

class DLLEXPORT_OST_GUI SpnavInput : public QThread {
  Q_OBJECT
public:
  ~SpnavInput();
  virtual void run();

  static SpnavInput* Instance();

  bool isValid() const {return valid_;}

signals:
  void deviceTransformed(int,int,int,int,int,int);
  void deviceButtonPressed(int);

private:
  SpnavInput(QObject* parent=NULL);
  bool valid_;
};

}} //ns

#endif
