//------------------------------------------------------------------------------
// This file is part of the OpenStructure project <www.openstructure.org>
//
// Copyright (C) 2008-2020 by the OpenStructure authors
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation; either version 3.0 of the License, or (at your option)
// any later version.
// This library is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this library; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//------------------------------------------------------------------------------
#include "tool_option_group.hh"

namespace ost { namespace gui {

ToolOptionGroup::ToolOptionGroup(const String& group_key, 
                                 const String& verbose_name):
  key_(group_key), verbose_name_(verbose_name)
{
  
}
  

void ToolOptionGroup::AddOption(ToolOption* tool_option)
{
  options_.push_back(tool_option);
}
  
const String& ToolOptionGroup::GetKey() const
{
  return key_;
}

const String& ToolOptionGroup::GetVerboseName() const
{
  return verbose_name_;
}

const ToolOptionList& ToolOptionGroup::GetOptions() const
{
  return options_;
}

ToolOption* ToolOptionGroup::FindOptionByKey(const String& key) const
{
  for (ToolOptionList::const_iterator i=options_.begin(), 
       e=options_.end(); i!=e; ++i) {
    if ((*i)->GetKey()==key) {
      return *i;
    }       
  }
  return NULL;
}

}}
