//------------------------------------------------------------------------------
// This file is part of the OpenStructure project <www.openstructure.org>
//
// Copyright (C) 2008-2020 by the OpenStructure authors
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation; either version 3.0 of the License, or (at your option)
// any later version.
// This library is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this library; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//------------------------------------------------------------------------------
#include "map.hh"

namespace ost{ namespace img {

MapHandle DLLEXPORT_OST_IMG_BASE CreateMap(const Extent& ext,
                        DataType type,
                        DataDomain dom)
{
  return CreateImage(ext,type,dom);
}

//! convenience variant to create images
MapHandle DLLEXPORT_OST_IMG_BASE CreateMap(const Size& s, const Point& o,
                                  DataType type,
                                  DataDomain dom)
{
  return CreateImage(s,o,type,dom);
}

MapHandle DLLEXPORT_OST_IMG_BASE CreateMap(const Point& p1, const Point& p2,
                                  DataType type,
                                  DataDomain dom)
{
  return CreateImage(p1,p2,type,dom);
}

MapHandle DLLEXPORT_OST_IMG_BASE CreateMap(const Point& p1, const Size& s,
                                  DataType type,
                                  DataDomain dom)
{
   return CreateImage(p1,s,type,dom);
}

}} // ns
