#declare _Helix = object {
mesh2 {
 vertex_vectors { 97,
  <19.9450,21.6440,89.4730>,
  <19.2122,21.5510,89.2156>,
  <18.4985,21.4830,88.9830>,
  <17.8231,21.4651,88.7998>,
  <17.2049,21.5222,88.6907>,
  <16.6632,21.6795,88.6806>,
  <16.2170,21.9620,88.7940>,
  <15.8831,22.3839,89.0421>,
  <15.6684,22.9164,89.3813>,
  <15.5778,23.5200,89.7545>,
  <15.6159,24.1552,90.1045>,
  <15.7874,24.7824,90.3740>,
  <16.0970,25.3620,90.5060>,
  <16.5388,25.8598,90.4574>,
  <17.0649,26.2628,90.2424>,
  <17.6169,26.5633,89.8893>,
  <18.1362,26.7535,89.4263>,
  <18.5644,26.8256,88.8818>,
  <18.8430,26.7720,88.2840>,
  <18.9272,26.5939,87.6618>,
  <18.8265,26.3281,87.0462>,
  <18.5640,26.0206,86.4685>,
  <18.1628,25.7171,85.9601>,
  <17.6461,25.4637,85.5524>,
  <17.0370,25.3060,85.2770>,
  <16.3629,25.2806,85.1554>,
  <15.6684,25.3858,85.1706>,
  <15.0024,25.6109,85.2954>,
  <14.4137,25.9447,85.5031>,
  <13.9513,26.3764,85.7666>,
  <13.6640,26.8950,86.0590>,
  <13.5866,27.4850,86.3529>,
  <13.6978,28.1125,86.6186>,
  <13.9618,28.7393,86.8259>,
  <14.3432,29.3268,86.9447>,
  <14.8065,29.8369,86.9448>,
  <15.3160,30.2310,86.7960>,
  <15.8358,30.4809,86.4794>,
  <16.3280,30.5987,86.0216>,
  <16.7540,30.6065,85.4601>,
  <17.0757,30.5264,84.8326>,
  <17.2545,30.3805,84.1771>,
  <17.2520,30.1910,83.5310>,
  <17.0433,29.9806,82.9295>,
  <16.6574,29.7745,82.3968>,
  <16.1366,29.5983,81.9543>,
  <15.5234,29.4779,81.6237>,
  <14.8600,29.4389,81.4264>,
  <14.1890,29.5070,81.3840>,
  <13.5506,29.7004,81.5069>,
  <12.9771,30.0070,81.7615>,
  <12.4985,30.4069,82.1030>,
  <12.1450,30.8804,82.4865>,
  <11.9468,31.4077,82.8675>,
  <11.9340,31.9690,83.2010>,
  <12.1232,32.5437,83.4480>,
  <12.4775,33.1072,83.5917>,
  <12.9462,33.6341,83.6212>,
  <13.4789,34.0990,83.5253>,
  <14.0250,34.4765,83.2929>,
  <14.5340,34.7410,82.9130>,
  <14.9608,34.8753,82.3850>,
  <15.2814,34.8946,81.7504>,
  <15.4776,34.8222,81.0609>,
  <15.5307,34.6815,80.3686>,
  <15.4223,34.4956,79.7253>,
  <15.1340,34.2880,79.1830>,
  <14.6602,34.0830,78.7828>,
  <14.0471,33.9094,78.5229>,
  <13.3538,33.7969,78.3911>,
  <12.6395,33.7753,78.3748>,
  <11.9632,33.8744,78.4615>,
  <11.3840,34.1240,78.6390>,
  <10.9510,34.5408,78.8907>,
  <10.6727,35.0895,79.1843>,
  <10.5476,35.7218,79.4835>,
  <10.5742,36.3894,79.7519>,
  <10.7508,37.0439,79.9532>,
  <11.0760,37.6370,80.0510>,
  <11.5391,38.1286,80.0171>,
  <12.0931,38.5112,79.8553>,
  <12.6818,38.7855,79.5778>,
  <13.2493,38.9522,79.1964>,
  <13.7394,39.0122,78.7231>,
  <14.0960,38.9660,78.1700>,
  <14.2759,38.8187,77.5536>,
  <14.2874,38.5922,76.9088>,
  <14.1516,38.3129,76.2750>,
  <13.8899,38.0069,75.6918>,
  <13.5233,37.7005,75.1987>,
  <13.0730,37.4200,74.8350>,
  <12.5585,37.1857,74.6290>,
  <11.9918,36.9949,74.5638>,
  <11.3836,36.8388,74.6113>,
  <10.7441,36.7087,74.7433>,
  <10.0837,36.5960,74.9316>,
  <9.4130,36.4920,75.1480>
 }
 normal_vectors { 97,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>
 }
 texture_list { 512,
  texture {_Helix_tex pigment {color rgbft <0.0000,0.0000,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.0000,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.0000,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.0000,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.0000,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.0000,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.0000,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.0000,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.0000,0.1429,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.1429,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.1429,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.1429,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.1429,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.1429,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.1429,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.1429,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.0000,0.2857,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.2857,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.2857,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.2857,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.2857,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.2857,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.2857,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.2857,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.0000,0.4286,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.4286,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.4286,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.4286,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.4286,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.4286,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.4286,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.4286,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.0000,0.5714,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.5714,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.5714,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.5714,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.5714,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.5714,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.5714,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.5714,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.0000,0.7143,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.7143,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.7143,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.7143,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.7143,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.7143,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.7143,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.7143,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.0000,0.8571,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.8571,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.8571,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.8571,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.8571,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.8571,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.8571,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.8571,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.0000,1.0000,0.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.0000,1.0000,0.1429,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.0000,1.0000,0.2857,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.0000,1.0000,0.4286,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.0000,1.0000,0.5714,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.0000,1.0000,0.7143,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.0000,1.0000,0.8571,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.0000,1.0000,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.1429,0.0000,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.0000,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.0000,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.0000,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.0000,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.0000,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.0000,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.0000,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.1429,0.1429,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.1429,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.1429,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.1429,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.1429,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.1429,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.1429,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.1429,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.1429,0.2857,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.2857,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.2857,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.2857,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.2857,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.2857,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.2857,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.2857,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.1429,0.4286,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.4286,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.4286,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.4286,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.4286,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.4286,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.4286,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.4286,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.1429,0.5714,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.5714,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.5714,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.5714,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.5714,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.5714,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.5714,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.5714,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.1429,0.7143,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.7143,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.7143,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.7143,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.7143,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.7143,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.7143,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.7143,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.1429,0.8571,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.8571,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.8571,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.8571,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.8571,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.8571,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.8571,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.8571,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.1429,1.0000,0.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.1429,1.0000,0.1429,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.1429,1.0000,0.2857,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.1429,1.0000,0.4286,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.1429,1.0000,0.5714,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.1429,1.0000,0.7143,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.1429,1.0000,0.8571,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.1429,1.0000,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.2857,0.0000,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.0000,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.0000,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.0000,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.0000,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.0000,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.0000,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.0000,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.2857,0.1429,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.1429,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.1429,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.1429,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.1429,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.1429,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.1429,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.1429,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.2857,0.2857,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.2857,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.2857,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.2857,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.2857,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.2857,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.2857,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.2857,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.2857,0.4286,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.4286,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.4286,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.4286,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.4286,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.4286,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.4286,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.4286,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.2857,0.5714,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.5714,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.5714,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.5714,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.5714,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.5714,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.5714,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.5714,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.2857,0.7143,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.7143,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.7143,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.7143,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.7143,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.7143,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.7143,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.7143,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.2857,0.8571,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.8571,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.8571,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.8571,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.8571,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.8571,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.8571,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.8571,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.2857,1.0000,0.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.2857,1.0000,0.1429,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.2857,1.0000,0.2857,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.2857,1.0000,0.4286,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.2857,1.0000,0.5714,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.2857,1.0000,0.7143,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.2857,1.0000,0.8571,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.2857,1.0000,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.4286,0.0000,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.0000,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.0000,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.0000,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.0000,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.0000,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.0000,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.0000,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.4286,0.1429,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.1429,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.1429,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.1429,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.1429,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.1429,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.1429,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.1429,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.4286,0.2857,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.2857,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.2857,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.2857,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.2857,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.2857,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.2857,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.2857,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.4286,0.4286,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.4286,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.4286,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.4286,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.4286,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.4286,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.4286,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.4286,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.4286,0.5714,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.5714,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.5714,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.5714,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.5714,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.5714,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.5714,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.5714,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.4286,0.7143,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.7143,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.7143,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.7143,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.7143,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.7143,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.7143,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.7143,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.4286,0.8571,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.8571,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.8571,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.8571,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.8571,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.8571,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.8571,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.8571,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.4286,1.0000,0.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.4286,1.0000,0.1429,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.4286,1.0000,0.2857,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.4286,1.0000,0.4286,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.4286,1.0000,0.5714,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.4286,1.0000,0.7143,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.4286,1.0000,0.8571,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.4286,1.0000,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.5714,0.0000,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.0000,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.0000,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.0000,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.0000,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.0000,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.0000,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.0000,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.5714,0.1429,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.1429,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.1429,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.1429,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.1429,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.1429,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.1429,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.1429,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.5714,0.2857,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.2857,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.2857,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.2857,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.2857,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.2857,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.2857,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.2857,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.5714,0.4286,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.4286,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.4286,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.4286,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.4286,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.4286,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.4286,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.4286,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.5714,0.5714,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.5714,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.5714,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.5714,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.5714,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.5714,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.5714,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.5714,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.5714,0.7143,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.7143,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.7143,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.7143,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.7143,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.7143,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.7143,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.7143,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.5714,0.8571,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.8571,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.8571,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.8571,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.8571,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.8571,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.8571,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.8571,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.5714,1.0000,0.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.5714,1.0000,0.1429,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.5714,1.0000,0.2857,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.5714,1.0000,0.4286,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.5714,1.0000,0.5714,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.5714,1.0000,0.7143,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.5714,1.0000,0.8571,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.5714,1.0000,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.7143,0.0000,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.0000,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.0000,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.0000,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.0000,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.0000,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.0000,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.0000,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.7143,0.1429,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.1429,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.1429,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.1429,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.1429,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.1429,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.1429,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.1429,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.7143,0.2857,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.2857,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.2857,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.2857,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.2857,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.2857,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.2857,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.2857,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.7143,0.4286,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.4286,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.4286,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.4286,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.4286,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.4286,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.4286,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.4286,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.7143,0.5714,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.5714,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.5714,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.5714,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.5714,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.5714,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.5714,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.5714,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.7143,0.7143,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.7143,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.7143,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.7143,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.7143,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.7143,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.7143,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.7143,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.7143,0.8571,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.8571,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.8571,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.8571,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.8571,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.8571,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.8571,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.8571,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.7143,1.0000,0.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.7143,1.0000,0.1429,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.7143,1.0000,0.2857,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.7143,1.0000,0.4286,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.7143,1.0000,0.5714,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.7143,1.0000,0.7143,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.7143,1.0000,0.8571,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.7143,1.0000,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.8571,0.0000,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.0000,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.0000,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.0000,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.0000,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.0000,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.0000,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.0000,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.8571,0.1429,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.1429,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.1429,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.1429,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.1429,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.1429,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.1429,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.1429,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.8571,0.2857,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.2857,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.2857,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.2857,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.2857,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.2857,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.2857,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.2857,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.8571,0.4286,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.4286,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.4286,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.4286,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.4286,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.4286,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.4286,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.4286,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.8571,0.5714,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.5714,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.5714,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.5714,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.5714,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.5714,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.5714,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.5714,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.8571,0.7143,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.7143,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.7143,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.7143,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.7143,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.7143,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.7143,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.7143,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.8571,0.8571,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.8571,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.8571,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.8571,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.8571,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.8571,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.8571,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.8571,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.8571,1.0000,0.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.8571,1.0000,0.1429,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.8571,1.0000,0.2857,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.8571,1.0000,0.4286,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.8571,1.0000,0.5714,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.8571,1.0000,0.7143,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.8571,1.0000,0.8571,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.8571,1.0000,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.0000,0.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.0000,0.1429,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.0000,0.2857,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.0000,0.4286,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.0000,0.5714,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.0000,0.7143,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.0000,0.8571,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.0000,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.1429,0.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.1429,0.1429,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.1429,0.2857,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.1429,0.4286,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.1429,0.5714,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.1429,0.7143,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.1429,0.8571,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.1429,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.2857,0.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.2857,0.1429,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.2857,0.2857,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.2857,0.4286,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.2857,0.5714,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.2857,0.7143,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.2857,0.8571,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.2857,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.4286,0.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.4286,0.1429,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.4286,0.2857,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.4286,0.4286,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.4286,0.5714,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.4286,0.7143,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.4286,0.8571,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.4286,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.5714,0.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.5714,0.1429,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.5714,0.2857,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.5714,0.4286,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.5714,0.5714,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.5714,0.7143,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.5714,0.8571,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.5714,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.7143,0.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.7143,0.1429,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.7143,0.2857,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.7143,0.4286,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.7143,0.5714,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.7143,0.7143,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.7143,0.8571,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.7143,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.8571,0.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.8571,0.1429,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.8571,0.2857,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.8571,0.4286,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.8571,0.5714,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.8571,0.7143,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.8571,0.8571,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.8571,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,1.0000,0.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,1.0000,0.1429,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,1.0000,0.2857,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,1.0000,0.4286,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,1.0000,0.5714,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,1.0000,0.7143,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,1.0000,0.8571,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,1.0000,1.0000,_Helix_fi,_Helix_tp>}}
 }
 face_indices { 0
 }
}
}
