//------------------------------------------------------------------------------
// This file is part of the OpenStructure project <www.openstructure.org>
//
// Copyright (C) 2008-2020 by the OpenStructure authors
// Copyright (C) 2003-2010 by the IPLT authors
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation; either version 3.0 of the License, or (at your option)
// any later version.
// This library is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this library; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//------------------------------------------------------------------------------

/*
  author: Andreas Schenk
*/

#include <cmath>

#include "phase.hh"


namespace ost { namespace img {

Phase::Phase(Real angle)
{
  angle_=reduce(angle);
}

Phase::Phase(Complex comp)
{
  Real angle=atan2(imag(comp),real(comp));
  angle_=reduce(angle);
}


Phase::operator Real () const
{
  return angle_;
}

const Phase Phase::operator+=(const Phase& rhs)
{
  angle_=reduce(angle_+static_cast<Real>(rhs));
  return Phase(*this);
}

const Phase Phase::operator-=(const Phase& rhs)
{
  angle_=reduce(angle_-static_cast<Real>(rhs));
  return Phase(*this);
}

Phase Phase::operator-() const
{
  return Phase(-angle_);
} 
  
const Phase Phase::operator*=(Real d)
{
  angle_=reduce(angle_*d);
  return *this;
}

const Phase Phase::operator/=(Real d)
{
  angle_=reduce(angle_/d);
  return *this;
}

Real Phase::reduce(Real in)
{
  return in-floor((in+M_PI)/(2.0*M_PI))*2*M_PI;
}

std::ostream& operator<<(std::ostream& o,const Phase a)
{
  o << static_cast<Real>(a);
  return o;
}


}} //ns
