#declare _Helix = object {
mesh2 {
 vertex_vectors { 17,
  <19.9450,21.6440,89.4730>,
  <16.2170,21.9620,88.7940>,
  <16.0970,25.3620,90.5060>,
  <18.8430,26.7720,88.2840>,
  <17.0370,25.3060,85.2770>,
  <13.6640,26.8950,86.0590>,
  <15.3160,30.2310,86.7960>,
  <17.2520,30.1910,83.5310>,
  <14.1890,29.5070,81.3840>,
  <11.9340,31.9690,83.2010>,
  <14.5340,34.7410,82.9130>,
  <15.1340,34.2880,79.1830>,
  <11.3840,34.1240,78.6390>,
  <11.0760,37.6370,80.0510>,
  <14.0960,38.9660,78.1700>,
  <13.0730,37.4200,74.8350>,
  <9.4130,36.4920,75.1480>
 }
 normal_vectors { 17,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>,
  <0.0000,0.0000,0.0000>
 }
 texture_list { 512,
  texture {_Helix_tex pigment {color rgbft <0.0000,0.0000,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.0000,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.0000,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.0000,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.0000,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.0000,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.0000,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.0000,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.0000,0.1429,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.1429,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.1429,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.1429,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.1429,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.1429,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.1429,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.1429,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.0000,0.2857,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.2857,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.2857,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.2857,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.2857,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.2857,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.2857,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.2857,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.0000,0.4286,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.4286,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.4286,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.4286,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.4286,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.4286,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.4286,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.4286,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.0000,0.5714,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.5714,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.5714,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.5714,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.5714,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.5714,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.5714,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.5714,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.0000,0.7143,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.7143,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.7143,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.7143,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.7143,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.7143,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.7143,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.7143,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.0000,0.8571,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.8571,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.8571,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.8571,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.8571,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.8571,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.8571,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.0000,0.8571,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.0000,1.0000,0.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.0000,1.0000,0.1429,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.0000,1.0000,0.2857,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.0000,1.0000,0.4286,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.0000,1.0000,0.5714,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.0000,1.0000,0.7143,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.0000,1.0000,0.8571,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.0000,1.0000,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.1429,0.0000,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.0000,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.0000,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.0000,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.0000,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.0000,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.0000,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.0000,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.1429,0.1429,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.1429,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.1429,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.1429,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.1429,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.1429,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.1429,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.1429,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.1429,0.2857,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.2857,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.2857,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.2857,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.2857,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.2857,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.2857,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.2857,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.1429,0.4286,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.4286,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.4286,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.4286,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.4286,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.4286,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.4286,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.4286,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.1429,0.5714,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.5714,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.5714,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.5714,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.5714,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.5714,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.5714,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.5714,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.1429,0.7143,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.7143,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.7143,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.7143,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.7143,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.7143,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.7143,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.7143,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.1429,0.8571,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.8571,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.8571,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.8571,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.8571,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.8571,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.8571,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.1429,0.8571,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.1429,1.0000,0.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.1429,1.0000,0.1429,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.1429,1.0000,0.2857,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.1429,1.0000,0.4286,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.1429,1.0000,0.5714,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.1429,1.0000,0.7143,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.1429,1.0000,0.8571,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.1429,1.0000,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.2857,0.0000,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.0000,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.0000,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.0000,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.0000,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.0000,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.0000,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.0000,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.2857,0.1429,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.1429,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.1429,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.1429,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.1429,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.1429,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.1429,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.1429,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.2857,0.2857,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.2857,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.2857,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.2857,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.2857,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.2857,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.2857,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.2857,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.2857,0.4286,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.4286,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.4286,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.4286,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.4286,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.4286,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.4286,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.4286,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.2857,0.5714,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.5714,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.5714,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.5714,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.5714,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.5714,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.5714,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.5714,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.2857,0.7143,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.7143,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.7143,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.7143,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.7143,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.7143,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.7143,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.7143,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.2857,0.8571,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.8571,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.8571,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.8571,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.8571,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.8571,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.8571,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.2857,0.8571,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.2857,1.0000,0.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.2857,1.0000,0.1429,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.2857,1.0000,0.2857,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.2857,1.0000,0.4286,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.2857,1.0000,0.5714,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.2857,1.0000,0.7143,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.2857,1.0000,0.8571,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.2857,1.0000,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.4286,0.0000,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.0000,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.0000,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.0000,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.0000,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.0000,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.0000,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.0000,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.4286,0.1429,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.1429,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.1429,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.1429,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.1429,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.1429,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.1429,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.1429,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.4286,0.2857,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.2857,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.2857,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.2857,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.2857,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.2857,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.2857,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.2857,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.4286,0.4286,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.4286,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.4286,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.4286,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.4286,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.4286,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.4286,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.4286,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.4286,0.5714,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.5714,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.5714,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.5714,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.5714,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.5714,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.5714,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.5714,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.4286,0.7143,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.7143,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.7143,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.7143,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.7143,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.7143,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.7143,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.7143,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.4286,0.8571,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.8571,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.8571,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.8571,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.8571,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.8571,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.8571,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.4286,0.8571,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.4286,1.0000,0.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.4286,1.0000,0.1429,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.4286,1.0000,0.2857,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.4286,1.0000,0.4286,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.4286,1.0000,0.5714,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.4286,1.0000,0.7143,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.4286,1.0000,0.8571,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.4286,1.0000,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.5714,0.0000,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.0000,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.0000,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.0000,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.0000,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.0000,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.0000,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.0000,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.5714,0.1429,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.1429,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.1429,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.1429,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.1429,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.1429,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.1429,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.1429,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.5714,0.2857,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.2857,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.2857,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.2857,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.2857,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.2857,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.2857,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.2857,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.5714,0.4286,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.4286,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.4286,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.4286,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.4286,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.4286,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.4286,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.4286,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.5714,0.5714,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.5714,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.5714,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.5714,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.5714,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.5714,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.5714,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.5714,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.5714,0.7143,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.7143,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.7143,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.7143,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.7143,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.7143,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.7143,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.7143,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.5714,0.8571,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.8571,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.8571,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.8571,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.8571,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.8571,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.8571,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.5714,0.8571,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.5714,1.0000,0.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.5714,1.0000,0.1429,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.5714,1.0000,0.2857,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.5714,1.0000,0.4286,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.5714,1.0000,0.5714,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.5714,1.0000,0.7143,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.5714,1.0000,0.8571,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.5714,1.0000,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.7143,0.0000,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.0000,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.0000,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.0000,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.0000,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.0000,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.0000,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.0000,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.7143,0.1429,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.1429,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.1429,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.1429,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.1429,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.1429,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.1429,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.1429,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.7143,0.2857,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.2857,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.2857,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.2857,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.2857,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.2857,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.2857,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.2857,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.7143,0.4286,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.4286,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.4286,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.4286,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.4286,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.4286,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.4286,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.4286,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.7143,0.5714,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.5714,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.5714,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.5714,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.5714,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.5714,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.5714,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.5714,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.7143,0.7143,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.7143,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.7143,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.7143,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.7143,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.7143,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.7143,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.7143,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.7143,0.8571,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.8571,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.8571,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.8571,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.8571,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.8571,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.8571,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.7143,0.8571,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.7143,1.0000,0.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.7143,1.0000,0.1429,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.7143,1.0000,0.2857,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.7143,1.0000,0.4286,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.7143,1.0000,0.5714,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.7143,1.0000,0.7143,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.7143,1.0000,0.8571,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.7143,1.0000,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.8571,0.0000,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.0000,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.0000,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.0000,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.0000,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.0000,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.0000,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.0000,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.8571,0.1429,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.1429,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.1429,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.1429,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.1429,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.1429,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.1429,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.1429,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.8571,0.2857,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.2857,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.2857,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.2857,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.2857,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.2857,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.2857,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.2857,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.8571,0.4286,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.4286,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.4286,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.4286,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.4286,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.4286,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.4286,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.4286,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.8571,0.5714,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.5714,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.5714,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.5714,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.5714,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.5714,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.5714,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.5714,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.8571,0.7143,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.7143,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.7143,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.7143,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.7143,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.7143,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.7143,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.7143,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.8571,0.8571,0.0000,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.8571,0.1429,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.8571,0.2857,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.8571,0.4286,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.8571,0.5714,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.8571,0.7143,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.8571,0.8571,_Helix_fi,_Helix_tp>}},
  texture {_Helix_tex pigment {color rgbft <0.8571,0.8571,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.8571,1.0000,0.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.8571,1.0000,0.1429,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.8571,1.0000,0.2857,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.8571,1.0000,0.4286,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.8571,1.0000,0.5714,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.8571,1.0000,0.7143,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.8571,1.0000,0.8571,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <0.8571,1.0000,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.0000,0.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.0000,0.1429,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.0000,0.2857,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.0000,0.4286,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.0000,0.5714,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.0000,0.7143,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.0000,0.8571,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.0000,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.1429,0.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.1429,0.1429,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.1429,0.2857,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.1429,0.4286,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.1429,0.5714,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.1429,0.7143,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.1429,0.8571,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.1429,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.2857,0.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.2857,0.1429,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.2857,0.2857,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.2857,0.4286,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.2857,0.5714,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.2857,0.7143,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.2857,0.8571,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.2857,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.4286,0.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.4286,0.1429,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.4286,0.2857,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.4286,0.4286,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.4286,0.5714,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.4286,0.7143,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.4286,0.8571,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.4286,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.5714,0.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.5714,0.1429,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.5714,0.2857,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.5714,0.4286,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.5714,0.5714,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.5714,0.7143,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.5714,0.8571,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.5714,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.7143,0.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.7143,0.1429,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.7143,0.2857,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.7143,0.4286,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.7143,0.5714,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.7143,0.7143,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.7143,0.8571,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.7143,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.8571,0.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.8571,0.1429,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.8571,0.2857,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.8571,0.4286,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.8571,0.5714,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.8571,0.7143,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.8571,0.8571,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,0.8571,1.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,1.0000,0.0000,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,1.0000,0.1429,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,1.0000,0.2857,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,1.0000,0.4286,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,1.0000,0.5714,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,1.0000,0.7143,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,1.0000,0.8571,_Helix_fi,_Helix_tp>}}
  texture {_Helix_tex pigment {color rgbft <1.0000,1.0000,1.0000,_Helix_fi,_Helix_tp>}}
 }
 face_indices { 0
 }
}
}
