//------------------------------------------------------------------------------
// This file is part of the OpenStructure project <www.openstructure.org>
//
// Copyright (C) 2008-2020 by the OpenStructure authors
//
// This library is free software; you can redistribute it and/or modify it under
// the terms of the GNU Lesser General Public License as published by the Free
// Software Foundation; either version 3.0 of the License, or (at your option)
// any later version.
// This library is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
// details.
//
// You should have received a copy of the GNU Lesser General Public License
// along with this library; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
//------------------------------------------------------------------------------
#include <boost/python.hpp>
using namespace boost::python;

//void export_Builder();
void export_Compound();
void export_Sanitizer();
void export_Conop();
void export_RingFinder();
void export_AminoAcids();
void export_processor();
void export_rule_based();
void export_heuristic();
void export_diag();

BOOST_PYTHON_MODULE(_ost_conop)
{
 // export_Builder();
  export_Conop();
  export_processor();
  export_rule_based();
  export_heuristic();
  export_Compound();
  export_RingFinder();
  export_AminoAcids();
  export_diag();
}
